@@ -21,7 +21,7 @@
 # Portions created by the Initial Developer are Copyright (C) 2008, 2009
 # the Initial Developer. All Rights Reserved.
 #
-# Contributor(s): Changwoo Ryu <cwryu@debian.org>
+# Contributor(s): See CREDITS file
 #
 # Alternatively, the contents of this file may be used under the terms of
 # either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -54,9 +54,9 @@ def V_NOT(jamos):
 def T_NOT(jamos):
     return ''.join([c for c in T_ALL if not c in jamos])
 
-V_A_O = V_A + V_O
-V_NOT_A_O = V_NOT(V_A + V_O)
-V_NOT_A_EO_O = V_NOT(V_A + V_EO + V_O)
+V_A_O = V_A + V_YA + V_O + V_YO
+V_NOT_A_O = V_NOT(V_A + V_YA + V_O + V_YO)
+V_NOT_A_EO_O = V_NOT(V_A + V_YA + V_EO + V_O + V_YO)
 
 # 조건
 
@@ -169,7 +169,12 @@ COND_EU_AO = [ u'[%s][%s]%s' % (V_A_O, L_ALL, V_EU),
 COND_EU_NOT_AO = [ u'[%s][%s]%s' % (V_NOT_A_O, L_ALL, V_EU),
                    u'[%s][%s][%s]%s' % (V_NOT_A_O, T_ALL, L_ALL, V_EU) ]
 
+#### 유성음/무성음 자모 구분
 
+T_VOICED = T_NIEUN + T_RIEUL + T_MIEUM + T_IEUNG
+T_UNVOICED = filter(lambda l: not l in T_VOICED, T_ALL)
+COND_VOICED = u'[%s]' % (V_ALL + T_VOICED)
+COND_UNVOICED = u'[%s]' % T_UNVOICED
 
 ######################################################################
 #### 어미 데이터
@@ -211,26 +216,26 @@ groups['-으시-'] = [
 groups['-었-'] = [
     { 'rules': [[u'-었-', COND_EOA_NOT_AO, ''],
                 [u'-았-', COND_EOA_AO, ''],
-                [u'-\u1165\u11bb-', COND_EOA_EO, V_EO],
-                [u'-\u1161\u11bb-', COND_EOA_A, V_A],
+                [u'-' + V_EO + T_SSANGSIOS + u'-', COND_EOA_EO, V_EO],
+                [u'-' + V_A + T_SSANGSIOS + u'-', COND_EOA_A, V_A],
                 [u'-였-', u'하', ''],
                 # 준말
-                [u'-\u116a\u11bb-', V_O, V_O], # 오았 -> 왔
-                [u'-\u116f\u11bb-', V_U, V_U], # 우었 -> 웠
-                [u'-\u116b\u11bb-', COND_EOA_OE, V_OE], # 외었 -> 왜ㅆ
-                [u'-\u116a\u11bb-', u'놓', V_O + T_HIEUH], # 놓아 -> 놔
-                [u'-\u1162\u11bb-', u'하', V_A], # 하였 -> 했
-                [u'-\u1167\u11bb-', V_I, V_I], # 이었 -> 였
-                [u'-\u11bb-', V_AE, ''], # 애었 -> 앴
-                [u'-\u11bb-', V_E, ''], # 에었 -> 엤
+                [u'-' + V_WA + T_SSANGSIOS + u'-', V_O, V_O], # 오았 -> 왔
+                [u'-' + V_WEO + T_SSANGSIOS + u'-', V_U, V_U], # 우었 -> 웠
+                [u'-' + V_WAE + T_SSANGSIOS + u'-', COND_EOA_OE, V_OE], # 외었 -> 왜ㅆ
+                [u'-' + V_WA + T_SSANGSIOS + u'-', u'놓', V_O + T_HIEUH], # 놓아 -> 놔
+                [u'-' + V_AE + T_SSANGSIOS + u'-', u'하', V_A], # 하였 -> 했
+                [u'-' + V_YEO + T_SSANGSIOS + u'-', V_I, V_I], # 이었 -> 였
+                [u'-' + T_SSANGSIOS + u'-', V_AE, ''], # 애었 -> 앴
+                [u'-' + T_SSANGSIOS + u'-', V_E, ''], # 에었 -> 엤
                 ],
       'after': ['#용언', '#이다', '-으시-'],
       'notcond': ['#ㄷ불규칙', '#ㅂ불규칙', '#ㅅ불규칙', '#ㅎ불규칙',
                   '#러불규칙', '#르불규칙', '#우불규칙', '#으불규칙'],
     },
     # ㄷ불규칙
-    { 'rules': [[u'-\u11af었-', '[%s]%s' % (V_NOT_A_O, T_TIKEUT), T_TIKEUT],
-                [u'-\u11af았-', '[%s]%s' % (V_A_O, T_TIKEUT), T_TIKEUT]],
+    { 'rules': [[u'-%s었-' % T_RIEUL, '[%s]%s' % (V_NOT_A_O, T_TIKEUT), T_TIKEUT],
+                [u'-%s았-' % T_RIEUL, '[%s]%s' % (V_A_O, T_TIKEUT), T_TIKEUT]],
       'after': ['#용언'],
       'cond': ['#ㄷ불규칙'],
     },
@@ -252,10 +257,10 @@ groups['-었-'] = [
       'cond': ['#ㅅ불규칙'],
     },
     # ㅎ불규칙
-    { 'rules': [[u'-\u1162\u11bb-', V_A + T_HIEUH, V_A + T_HIEUH],   # 파랗다
-                [u'-\u1164\u11bb-', V_YA + T_HIEUH, V_YA + T_HIEUH], # 하얗다
-                [u'-\u1166\u11bb-', V_EO + T_HIEUH, V_EO + T_HIEUH], # 누렇다
-                [u'-\u1168\u11bb-', V_YEO + T_HIEUH, V_YEO + T_HIEUH], # 허옇다
+    { 'rules': [[u'-' + V_AE + T_SSANGSIOS + u'-', V_A + T_HIEUH, V_A + T_HIEUH],   # 파랗다
+                [u'-' + V_YAE + T_SSANGSIOS + u'-', V_YA + T_HIEUH, V_YA + T_HIEUH], # 하얗다
+                [u'-' + V_E + T_SSANGSIOS + u'-', V_EO + T_HIEUH, V_EO + T_HIEUH], # 누렇다
+                [u'-' + V_YE + T_SSANGSIOS + u'-', V_YEO + T_HIEUH, V_YEO + T_HIEUH], # 허옇다
                ],
       'after': ['#용언'],
       'notafter': ['그렇다', '고렇다', '이렇다', '요렇다', '저렇다', '조렇다',
@@ -263,7 +268,7 @@ groups['-었-'] = [
       'cond': ['#ㅎ불규칙'],
     },
     # ㅎ불규칙 지시형용사
-    { 'rules': [[u'-\u1162\u11bb-', V_EO + T_HIEUH, V_EO + T_HIEUH]],
+    { 'rules': [[u'-' + V_AE + T_SSANGSIOS + u'-', V_EO + T_HIEUH, V_EO + T_HIEUH]],
       'after': ['그렇다', '고렇다', '이렇다', '요렇다', '저렇다', '조렇다',
                 '어떻다', '아무렇다'],
       'cond': ['#ㅎ불규칙'],
@@ -274,25 +279,25 @@ groups['-었-'] = [
       'cond': ['#러불규칙'],
     },
     # 르불규칙
-    { 'rules': [[u'-\u11af렀-', COND_REU_NOT_AO, u'르'],
-                [u'-\u11af랐-', COND_REU_AO, u'르']],
+    { 'rules': [[u'-%s렀-' % T_RIEUL, COND_REU_NOT_AO, u'르'],
+                [u'-%s랐-' % T_RIEUL, COND_REU_AO, u'르']],
       'after': ['#용언'],
       'cond': ['#르불규칙'],
     },
     # 우불규칙
-    { 'rules': [[u'-\u1165\u11bb-', V_U, V_U]],
+    { 'rules': [[u'-' + V_EO + T_SSANGSIOS + u'-', V_U, V_U]],
       'after': ['#용언'],
       'cond': ['#우불규칙'],
     },
     # 으불규칙
-    { 'rules': [[u'-\u1161\u11bb-', COND_EU_AO, V_EU],
-                [u'-\u1165\u11bb-', COND_EU_NOT_AO, V_EU]],
+    { 'rules': [[u'-' + V_A + T_SSANGSIOS + u'-', COND_EU_AO, V_EU],
+                [u'-' + V_EO + T_SSANGSIOS + u'-', COND_EU_NOT_AO, V_EU]],
       'after': ['#용언'],
       'notafter': ['^.*끄다$', '^.*뜨다$', '^.*쓰다$', '^.*크다$'],
       'cond': ['#으불규칙'],
     },
     # 으불규칙 예외 '끄다', '뜨다', '쓰다', '크다'
-    { 'rules': [[u'-\u1165\u11bb-', V_EU, V_EU]],
+    { 'rules': [[u'-' + V_EO + T_SSANGSIOS + u'-', V_EU, V_EU]],
       'after': ['^.*끄다$', '^.*뜨다$', '^.*쓰다$', '^.*크다$'],
       'cond': ['#으불규칙'],
     },
@@ -325,25 +330,25 @@ groups['-더-'] = [
 groups['-어'] = [
     { 'rules': [[u'-어', COND_EOA_NOT_AO, ''],
                 [u'-아', COND_EOA_AO, ''],
-                [u'-\u1165', COND_EOA_EO, V_EO],
-                [u'-\u1161', COND_EOA_A, V_A],
+                [u'-' + V_EO, COND_EOA_EO, V_EO],
+                [u'-' + V_A, COND_EOA_A, V_A],
                 [u'-여', u'하', ''],
-                [u'-\u116a', V_O, V_O], # 오아 -> 와
-                [u'-\u116f', V_U, V_U], # 우어 -> 워
-                [u'-\u116b', COND_EOA_OE, V_OE], # 외어 -> 왜
-                [u'-\u116a', u'놓', V_O + T_HIEUH], # 놓아 -> 놔
-                [u'-\u1162', u'하', V_A], # 하여 -> 해
-                [u'-\u1167', V_I, V_I], # 이어 -> 여
-                [u'-\u1162', V_AE, V_AE], # 애어 -> 애
-                [u'-\u1166', V_E, V_E], # 에어 -> 에
+                [u'-' + V_WA, V_O, V_O], # 오아 -> 와
+                [u'-' + V_WEO, V_U, V_U], # 우어 -> 워
+                [u'-' + V_WAE, COND_EOA_OE, V_OE], # 외어 -> 왜
+                [u'-' + V_WA, u'놓', V_O + T_HIEUH], # 놓아 -> 놔
+                [u'-' + V_AE, u'하', V_A], # 하여 -> 해
+                [u'-' + V_YEO, V_I, V_I], # 이어 -> 여
+                [u'-' + V_AE, V_AE, V_AE], # 애어 -> 애
+                [u'-' + V_E, V_E, V_E], # 에어 -> 에
                 ],
       'after': ['#용언', '-었-', '-겠-', '-으시-'],
       'notcond': ['#ㄷ불규칙', '#ㅂ불규칙', '#ㅅ불규칙', '#ㅎ불규칙',
                   '#러불규칙', '#르불규칙', '#우불규칙', '#으불규칙'],
     },
     # ㄷ불규칙
-    { 'rules': [[u'-\u11af어', '[%s]%s' % (V_NOT_A_O, T_TIKEUT), T_TIKEUT],
-                [u'-\u11af아', '[%s]%s' % (V_A_O, T_TIKEUT), T_TIKEUT]],
+    { 'rules': [[u'-%s어' % T_RIEUL, '[%s]%s' % (V_NOT_A_O, T_TIKEUT), T_TIKEUT],
+                [u'-%s아' % T_RIEUL, '[%s]%s' % (V_A_O, T_TIKEUT), T_TIKEUT]],
       'after': ['#용언'],
       'cond': ['#ㄷ불규칙'],
     },
@@ -365,10 +370,10 @@ groups['-어'] = [
       'cond': ['#ㅅ불규칙'],
     },
     # ㅎ불규칙
-    { 'rules': [[u'-\u1162', V_A + T_HIEUH, V_A + T_HIEUH],   # 파랗다
-                [u'-\u1164', V_YA + T_HIEUH, V_YA + T_HIEUH], # 하얗다
-                [u'-\u1166', V_EO + T_HIEUH, V_EO + T_HIEUH], # 누렇다
-                [u'-\u1168', V_YEO + T_HIEUH, V_YEO + T_HIEUH], # 허옇다
+    { 'rules': [[u'-' + V_AE, V_A + T_HIEUH, V_A + T_HIEUH],   # 파랗다
+                [u'-' + V_YAE, V_YA + T_HIEUH, V_YA + T_HIEUH], # 하얗다
+                [u'-' + V_E, V_EO + T_HIEUH, V_EO + T_HIEUH], # 누렇다
+                [u'-' + V_YE, V_YEO + T_HIEUH, V_YEO + T_HIEUH], # 허옇다
                ],
       'after': ['#용언'],
       'notafter': ['그렇다', '고렇다', '이렇다', '요렇다', '저렇다', '조렇다',
@@ -376,7 +381,7 @@ groups['-어'] = [
       'cond': ['#ㅎ불규칙'],
     },
     # ㅎ불규칙 지시형용사
-    { 'rules': [[u'-\u1162', V_EO + T_HIEUH, V_EO + T_HIEUH]],
+    { 'rules': [[u'-' + V_AE, V_EO + T_HIEUH, V_EO + T_HIEUH]],
       'after': ['그렇다', '고렇다', '이렇다', '요렇다', '저렇다', '조렇다',
                 '어떻다', '아무렇다'],
       'cond': ['#ㅎ불규칙'],
@@ -387,25 +392,25 @@ groups['-어'] = [
       'cond': ['#러불규칙'],
     },
     # 르불규칙
-    { 'rules': [[u'-\u11af러', COND_REU_NOT_AO, u'르'],
-                [u'-\u11af라', COND_REU_AO, u'르']],
+    { 'rules': [[u'-%s러' % T_RIEUL, COND_REU_NOT_AO, u'르'],
+                [u'-%s라' % T_RIEUL, COND_REU_AO, u'르']],
       'after': ['#용언'],
       'cond': ['#르불규칙'],
     },
     # 우불규칙
-    { 'rules': [[u'-\u1165', V_U, V_U]],
+    { 'rules': [[u'-' + V_EO, V_U, V_U]],
       'after': ['#용언'],
       'cond': ['#우불규칙'],
     },
     # 으불규칙
-    { 'rules': [[u'-\u1161', COND_EU_AO, V_EU],
-                [u'-\u1165', COND_EU_NOT_AO, V_EU]],
+    { 'rules': [[u'-' + V_A, COND_EU_AO, V_EU],
+                [u'-' + V_EO, COND_EU_NOT_AO, V_EU]],
       'after': ['#용언'],
       'notafter': ['^.*끄다$', '^.*뜨다$', '^.*쓰다$', '^.*크다$'],
       'cond': ['#으불규칙'],
     },
     # 으불규칙 예외 '끄다', '뜨다', '크다', '쓰다'
-    { 'rules': [[u'-\u1165', V_EU, V_EU]],
+    { 'rules': [[u'-' + V_EO, V_EU, V_EU]],
       'after': ['^.*끄다$', '^.*뜨다$', '^.*쓰다$', '^.*크다$'],
       'cond': ['#으불규칙'],
     },
@@ -466,7 +471,7 @@ for klass in groups['-어서']:
     for r in klass['rules']:
         r[0] = r[0] + '서'
 groups['-어서'][0]['after'].append('#이다')
-attach_emphasis(groups['-어서'], ['는', '도', '요'])
+attach_emphasis(groups['-어서'], ['는', T_NIEUN, '도', '요'])
 
 #### 연결: -어야, -아야
 # '-어' 재활용
@@ -515,14 +520,14 @@ groups['-어요'] += [
     # 이다/아니다의 경우 -에요 가능, 줄임 이예요 => 예요
     { 'rules': [[u'-에요', V_I, ''],
                 [u'-%s요' % V_YE, V_I, V_I]],
-      'after': ['#이다', '^.*아니다$'],
+      'after': ['#이다', '아니다'],
     }
 ]
 
 #### 관형사형 전성: -ㄹ, -을
 groups['-을'] = [
-    { 'rules': [[u'-\u11af', COND_V_ALL, ''],
-                [u'-\u11af', T_RIEUL, T_RIEUL],
+    { 'rules': [[u'-' + T_RIEUL, COND_V_ALL, ''],
+                [u'-' + T_RIEUL, T_RIEUL, T_RIEUL],
                 [u'-을', COND_T_NOT_RIEUL, '']],
       'after': ['#용언', '#이다', '-으시-', '-시오-', '-었-'],
       'notcond': ['#ㄷ불규칙', '#ㅂ불규칙', '#ㅅ불규칙', '#ㅎ불규칙'],
@@ -534,7 +539,7 @@ groups['-을'] = [
     # ㅅ불규칙
     SIOS_IRREGULAR_TYPICAL_CLASS(u'-을', ['#용언']),
     # ㅎ불규칙
-    HIEUH_IRREGULAR_TYPICAL_CLASS(u'-\u11af', ['#용언']),
+    HIEUH_IRREGULAR_TYPICAL_CLASS(u'-' + T_RIEUL, ['#용언']),
 ]
 
 #### 종결: -ㄹ걸, -을걸
@@ -547,8 +552,8 @@ for klass in groups['-을걸']:
 #### 종결: -ㄹ게, -을게
 # 동사 전용
 groups['-을게'] = [
-    { 'rules': [[u'-\u11af게', COND_V_ALL, ''],
-                [u'-\u11af게', T_RIEUL, T_RIEUL],
+    { 'rules': [[u'-%s게' % T_RIEUL, COND_V_ALL, ''],
+                [u'-%s게' % T_RIEUL, T_RIEUL, T_RIEUL],
                 [u'-을게', COND_T_NOT_RIEUL, '']],
       'after': ['#동사'],
       'notcond': ['#ㄷ불규칙', '#ㅂ불규칙', '#ㅅ불규칙'],
@@ -592,7 +597,7 @@ groups['-을지'] = copy_group(groups['-을'])
 for klass in groups['-을지']:
     for r in klass['rules']:
         r[0] = r[0] + '지'
-attach_emphasis(groups['-을지'], ['는', '도'])
+attach_emphasis(groups['-을지'], ['는', T_NIEUN, '도'])
 
 #### 연결: -ㄹ지라도, -을지라도
 # '-을' 재활용
@@ -627,7 +632,8 @@ groups['-는'] = [
 #### 연결: -게
 groups['-게'] = [
     { 'rules': [[u'-게', '', ''],
-                [u'-케', u'하', u'하'], # 하게 -> 케 준말
+                [u'-케', COND_VOICED + u'하', u'하'], # 하게 -> 케 준말
+                [u'-게', COND_UNVOICED + u'하', u'하'], # 하게 -> 게 준말
                 ],
       'after': ['#용언', '-으시-'],
     },
@@ -643,8 +649,8 @@ groups['-다가'] = [
 
 #### 관형사형 전성: -ㄴ, -은
 groups['-은'] = [
-    { 'rules': [[u'-\u11ab', COND_V_ALL, ''],
-                [u'-\u11ab', T_RIEUL, T_RIEUL],
+    { 'rules': [[u'-' + T_NIEUN, COND_V_ALL, ''],
+                [u'-' + T_NIEUN, T_RIEUL, T_RIEUL],
                 [u'-은', COND_T_NOT_RIEUL, '']],
       'after': ['#용언', '#이다', '-으시-'],
       'notafter': ['^.*있다$', '^.*없다$'],
@@ -657,7 +663,7 @@ groups['-은'] = [
     # ㅅ불규칙
     SIOS_IRREGULAR_TYPICAL_CLASS(u'-은', ['#용언']),
     # ㅎ불규칙
-    HIEUH_IRREGULAR_TYPICAL_CLASS(u'-\u11ab', ['#용언']),
+    HIEUH_IRREGULAR_TYPICAL_CLASS(u'-' + T_NIEUN, ['#용언']),
 ]
 
 #### 연결: -지
@@ -669,7 +675,7 @@ groups['-지'] = [
       'after': ['#용언', '#이다', '-으시-', '-었-', '-겠-'],
     },
 ]
-attach_emphasis(groups['-지'], ['는', '도', '요'])
+attach_emphasis(groups['-지'], ['는', T_NIEUN, '도', '요'])
 # 지요 -> 죠 준말
 groups['-지'][0]['rules'].append([u'-죠', '', ''])
 
@@ -704,8 +710,8 @@ groups['-으며'] = [
 
 #### 종결: -ㅂ니다, -습니다
 groups['-습니다'] = [
-    { 'rules': [[u'-\u11b8니다', COND_V_ALL, ''],
-                [u'-\u11b8니다', T_RIEUL, T_RIEUL],
+    { 'rules': [[u'-%s니다' % T_PIEUP, COND_V_ALL, ''],
+                [u'-%s니다' % T_PIEUP, T_RIEUL, T_RIEUL],
                 [u'-습니다', COND_T_NOT_RIEUL, '']],
       'after': ['#용언', '#이다', '-으시-', '-었-', '-겠-'],
     },
@@ -713,8 +719,8 @@ groups['-습니다'] = [
 
 #### 종결: -ㅂ니까, -습니까
 groups['-습니까'] = [
-    { 'rules': [[u'-\u11b8니까', COND_V_ALL, ''],
-                [u'-\u11b8니까', T_RIEUL, T_RIEUL],
+    { 'rules': [[u'-%s니까' % T_PIEUP, COND_V_ALL, ''],
+                [u'-%s니까' % T_PIEUP, T_RIEUL, T_RIEUL],
                 [u'-습니까', COND_T_NOT_RIEUL, '']],
       'after': ['#용언', '#이다', '-으시-', '-었-', '-겠-'],
     },
@@ -877,8 +883,8 @@ groups['-다시피'] = [
 
 #### 종결: -ㅂ시다, -읍시다
 groups['-읍시다'] = [
-    { 'rules': [[u'-\u11b8시다', COND_V_ALL, ''],
-                [u'-\u11b8시다', T_RIEUL, T_RIEUL],
+    { 'rules': [[u'-%s시다' % T_PIEUP, COND_V_ALL, ''],
+                [u'-%s시다' % T_PIEUP, T_RIEUL, T_RIEUL],
                 [u'-읍시다', COND_T_NOT_RIEUL, '']],
       'after': ['#동사', '-으시-'],
       'notcond': ['#ㄷ불규칙', '#ㅂ불규칙', '#ㅅ불규칙'],
@@ -984,7 +990,9 @@ attach_emphasis(groups['-으라고'], ['요'])
 #### 연결: -라는, -으라는 (라고 하는)
 groups['-으라는'] = [
     { 'rules': [[u'-라는', COND_V_OR_RIEUL, ''],
-                [u'-으라는', COND_T_NOT_RIEUL, '']],
+                [u'-으라는', COND_T_NOT_RIEUL, ''],
+                [u'-란', COND_V_OR_RIEUL, ''],
+                [u'-으란', COND_T_NOT_RIEUL, '']],
       'after': ['#동사', '#이다', '아니다', '-으시-', '-더-', '-으리-'],
       'notcond': ['#ㄷ불규칙', '#ㅂ불규칙', '#ㅅ불규칙'],
     },
@@ -1005,18 +1013,18 @@ groups['-으래도'] = [
       'notcond': ['#ㄷ불규칙', '#ㅂ불규칙', '#ㅅ불규칙'],
     },
     # ㄷ불규칙
-    TIKEUT_IRREGULAR_TYPICAL_CLASS(u'-으라는', ['#동사']),
+    TIKEUT_IRREGULAR_TYPICAL_CLASS(u'-으래도', ['#동사']),
     # ㅂ불규칙
-    PIEUP_IRREGULAR_TYPICAL_CLASS(u'-우라는', ['#동사']),
+    PIEUP_IRREGULAR_TYPICAL_CLASS(u'-우래도', ['#동사']),
     # ㅅ불규칙
-    SIOS_IRREGULAR_TYPICAL_CLASS(u'-으라는', ['#동사']),
+    SIOS_IRREGULAR_TYPICAL_CLASS(u'-으래도', ['#동사']),
     # 동사이므로 ㅎ불규칙 해당 없음
 ]
 
 #### 연결 -ㄴ데, -은데
 groups['-은데'] = [
-    { 'rules': [[u'-\u11ab데', COND_V_OR_RIEUL, ''],
-                [u'-\u11ab데', T_RIEUL, T_RIEUL],
+    { 'rules': [[u'-%s데' % T_NIEUN, COND_V_OR_RIEUL, ''],
+                [u'-%s데' % T_NIEUN, T_RIEUL, T_RIEUL],
                 [u'-은데', COND_T_NOT_RIEUL, '']],
       'after': ['#형용사', '#이다', '-으시-', '-사오-'],
       'notafter': ['^.*있다$', '^.*없다$'],
@@ -1047,8 +1055,8 @@ attach_continuation_flags(groups['-기'], [flags.josa_ida_flag] + range(flags.jo
 
 #### 명사형 전성: -음
 groups['-음'] = [
-    { 'rules': [[u'-\u11b7', COND_V_ALL, ''],
-                [u'-\u11b1', T_RIEUL, T_RIEUL],
+    { 'rules': [[u'-' + T_MIEUM, COND_V_ALL, ''],
+                [u'-' + T_RIEUL_MIEUM, T_RIEUL, T_RIEUL],
                 [u'-음', COND_T_NOT_RIEUL, '']],
       'after': ['#용언', '#이다', '-으시-', '-었-', '-겠-'],
       'notcond': ['#ㄷ불규칙', '#ㅂ불규칙', '#ㅅ불규칙', '#ㅎ불규칙'],
@@ -1060,7 +1068,7 @@ groups['-음'] = [
     # ㅅ불규칙
     SIOS_IRREGULAR_TYPICAL_CLASS(u'-음', ['#용언']),
     # ㅎ불규칙
-    HIEUH_IRREGULAR_TYPICAL_CLASS(u'-\u11b7', ['#용언']),
+    HIEUH_IRREGULAR_TYPICAL_CLASS(u'-' + T_MIEUM, ['#용언']),
 ]
 # 조사
 # FIXME: 일단 모든 조사 허용
@@ -1068,8 +1076,8 @@ attach_continuation_flags(groups['-음'], [flags.josa_ida_flag] + range(flags.jo
 
 #### 종결: -ㄴ다, -는다
 groups['-는다'] = [
-    { 'rules': [[u'-\u11ab다', COND_V_OR_RIEUL, ''],
-                [u'-\u11ab다', T_RIEUL, T_RIEUL],
+    { 'rules': [[u'-%s다' % T_NIEUN, COND_V_OR_RIEUL, ''],
+                [u'-%s다' % T_NIEUN, T_RIEUL, T_RIEUL],
                 [u'-는다', COND_T_NOT_RIEUL, '']],
       'after': ['#동사', '-으시-'],
     },
@@ -1077,8 +1085,8 @@ groups['-는다'] = [
 
 #### 연결: -ㄴ다고, -는다고
 groups['-는다고'] = [
-    { 'rules': [[u'-\u11ab다고', COND_V_ALL, ''],
-                [u'-\u11ab다고', T_RIEUL, T_RIEUL],
+    { 'rules': [[u'-%s다고' % T_NIEUN, COND_V_ALL, ''],
+                [u'-%s다고' % T_NIEUN, T_RIEUL, T_RIEUL],
                 [u'-는다고', COND_T_NOT_RIEUL, '']],
       'after': ['#동사', '-으시-'],
     },
@@ -1086,8 +1094,8 @@ groups['-는다고'] = [
 
 #### 연결: -ㄴ다는, -는다는 (는다고 하는)
 groups['-는다는'] = [
-    { 'rules': [[u'-\u11ab다는', COND_V_ALL, ''],
-                [u'-\u11ab다는', T_RIEUL, T_RIEUL],
+    { 'rules': [[u'-%s다는' % T_NIEUN, COND_V_ALL, ''],
+                [u'-%s다는' % T_NIEUN, T_RIEUL, T_RIEUL],
                 [u'-는다는', COND_T_NOT_RIEUL, '']],
       'after': ['#동사', '-으시-'],
     },
@@ -1095,8 +1103,8 @@ groups['-는다는'] = [
 
 #### 연결: -ㄴ단, -는단 (는다고 하는, 는다고 한)
 groups['-는단'] = [
-    { 'rules': [[u'-\u11ab단', COND_V_ALL, ''],
-                [u'-\u11ab단', T_RIEUL, T_RIEUL],
+    { 'rules': [[u'-%s단' % T_NIEUN, COND_V_ALL, ''],
+                [u'-%s단' % T_NIEUN, T_RIEUL, T_RIEUL],
                 [u'-는단', COND_T_NOT_RIEUL, '']],
       'after': ['#동사', '-으시-'],
     },
@@ -1104,8 +1112,8 @@ groups['-는단'] = [
 
 #### 연결: -ㄴ다면, -는다면
 groups['-는다면'] = [
-    { 'rules': [[u'-\u11ab다면', COND_V_ALL, ''],
-                [u'-\u11ab다면', T_RIEUL, T_RIEUL],
+    { 'rules': [[u'-%s다면' % T_NIEUN, COND_V_ALL, ''],
+                [u'-%s다면' % T_NIEUN, T_RIEUL, T_RIEUL],
                 [u'-는다면', COND_T_NOT_RIEUL, '']],
       'after': ['#동사', '-으시-'],
     },
@@ -1113,11 +1121,11 @@ groups['-는다면'] = [
 
 #### 종결: -ㄴ다면서, -는다면서
 groups['-는다면서'] = [
-    { 'rules': [[u'-\u11ab다면서', COND_V_ALL, ''],
-                [u'-\u11ab다면서', T_RIEUL, T_RIEUL],
+    { 'rules': [[u'-%s다면서' % T_NIEUN, COND_V_ALL, ''],
+                [u'-%s다면서' % T_NIEUN, T_RIEUL, T_RIEUL],
                 [u'-는다면서', COND_T_NOT_RIEUL, ''],
-                [u'-\u11ab다며', COND_V_ALL, ''],
-                [u'-\u11ab다며', T_RIEUL, T_RIEUL],
+                [u'-%s다며' % T_NIEUN, COND_V_ALL, ''],
+                [u'-%s다며' % T_NIEUN, T_RIEUL, T_RIEUL],
                 [u'-는다며', COND_T_NOT_RIEUL, ''],
                 ],
       'after': ['#동사', '-으시-'],
@@ -1149,7 +1157,7 @@ groups['-는지'] = [
       'after': ['#동사', '^.*있다$', '^.*없다$', '^.*계시다$', '-으시-', '-었-', '-겠-'],
     },
 ]
-attach_emphasis(groups['-는지'], ['도', '는', '요'])
+attach_emphasis(groups['-는지'], ['는', T_NIEUN, '도', '요'])
 
 #### 연결: -면, -으면
 groups['-으면'] = [
@@ -1195,8 +1203,8 @@ groups['-던'] = [
 #### 연결: -ㄴ가, -은가
 #### 의문형 종결: -ㄴ가, -은가
 groups['-은가'] = [
-    { 'rules': [[u'-\u11ab가', COND_V_ALL, ''],
-                [u'-\u11ab가', T_RIEUL, T_RIEUL],
+    { 'rules': [[u'-%s가' % T_NIEUN, COND_V_ALL, ''],
+                [u'-%s가' % T_NIEUN, T_RIEUL, T_RIEUL],
                 [u'-은가', COND_T_NOT_RIEUL, '']],
       'after': ['#형용사', '#이다', '-으시-'],
       'notafter': ['^.*있다$', '^.*없다$'],
@@ -1209,7 +1217,7 @@ groups['-은가'] = [
     # ㅅ불규칙
     SIOS_IRREGULAR_TYPICAL_CLASS(u'-은가', ['#형용사']),
     # ㅎ불규칙
-    HIEUH_IRREGULAR_TYPICAL_CLASS(u'-\u11ab가', ['#형용사']),
+    HIEUH_IRREGULAR_TYPICAL_CLASS(u'-%s가' % T_NIEUN, ['#형용사']),
 ]
 attach_emphasis(groups['-은가'], ['요'])
 
@@ -1248,8 +1256,8 @@ groups['-로구나'] = [
 
 #### 연결: -ㄴ지, -은지
 groups['-은지'] = [
-    { 'rules': [[u'-\u11ab지', COND_V_ALL, ''],
-                [u'-\u11ab지', T_RIEUL, T_RIEUL],
+    { 'rules': [[u'-%s지' % T_NIEUN, COND_V_ALL, ''],
+                [u'-%s지' % T_NIEUN, T_RIEUL, T_RIEUL],
                 [u'-은지', COND_T_NOT_RIEUL, '']],
       'after': ['#이다', '#형용사', '-으시-'],
       'notafter': ['^.*있다$', '^.*없다$'],
@@ -1262,9 +1270,9 @@ groups['-은지'] = [
     # ㅅ불규칙
     SIOS_IRREGULAR_TYPICAL_CLASS(u'-은지', ['#형용사']),
     # ㅎ불규칙
-    HIEUH_IRREGULAR_TYPICAL_CLASS(u'-\u11ab지', ['#형용사']),
+    HIEUH_IRREGULAR_TYPICAL_CLASS(u'-%s지' % T_NIEUN, ['#형용사']),
 ]
-attach_emphasis(groups['-은지'], ['도', '는', '요'])
+attach_emphasis(groups['-은지'], ['는', T_NIEUN, '도', '요'])
 
 #### 종결: -십시오, -으십시오
 groups['-으십시오'] = [
@@ -1437,6 +1445,7 @@ for klass in groups['-으냐고']:
     for r in klass['rules']:
         new_rule.append([r[0] + '고'] + r[1:])
     klass['rules'] = new_rule
+attach_emphasis(groups['-으냐고'], ['요'])
 
 #### 연결: -으냐네 (-으냐고 하네)
 groups['-으냐네'] = [k.copy() for k in groups['-으냐']]
@@ -1509,6 +1518,7 @@ for klass in groups['-느냐고']:
     for r in klass['rules']:
         new_rule.append([r[0] + '고'] + r[1:])
     klass['rules'] = new_rule
+attach_emphasis(groups['-느냐고'], ['요'])
 
 #### 연결: -느냐네 (-느냐고 하네)
 groups['-느냐네'] = [k.copy() for k in groups['-느냐']]
@@ -1719,7 +1729,7 @@ groups['-으라네'] = [
     { 'rules': [[u'-라네', COND_V_ALL, ''],
                 [u'-라네', T_RIEUL, T_RIEUL],
                 [u'-으라네', COND_T_NOT_RIEUL, '']],
-      'after': ['#동사', '#이다', '^.*아니다$', '-으시-', '-더-', '-으리-'],
+      'after': ['#동사', '#이다', '아니다', '-으시-', '-더-', '-으리-'],
       'notcond': ['#ㄷ불규칙', '#ㅂ불규칙', '#ㅅ불규칙'],
     },
     # ㄷ불규칙
@@ -1766,7 +1776,7 @@ groups['-을는지'] = [
     # ㅎ불규칙
     HIEUH_IRREGULAR_TYPICAL_CLASS(u'-%s는지' % T_RIEUL, ['#용언']),
 ]
-attach_emphasis(groups['-을는지'], ['는', '도'])
+attach_emphasis(groups['-을는지'], ['는', T_NIEUN, '도'])
 
 #### 종결: -ㄴ다니, -는다니
 groups['-는다니'] = [
@@ -1826,3 +1836,71 @@ groups['-을래'] = [
 ]
 attach_emphasis(groups['-을래'], ['요'])
 
+#### 종결: -답니다
+groups['-답니다'] = [
+    { 'rules': [[u'-답니다', '', '']],
+      'after': ['#형용사', '-으시-', '-었-', '-겠-'],
+    },
+]
+
+#### 종결: -는답니다
+groups['-는답니다'] = [
+    { 'rules': [[u'-%s답니다' % T_NIEUN, COND_V_ALL, ''],
+                [u'-%s답니다' % T_NIEUN, T_RIEUL, T_RIEUL],
+                [u'-는답니다', COND_T_NOT_RIEUL, '']],
+      'after': ['#동사', '-으시-'],
+    },
+]
+
+#### 종결: -ㄹ세, -을세
+groups['-을세'] = [
+    { 'rules': [[u'-%s세' % T_RIEUL, COND_V_ALL, ''],
+                [u'-%s세' % T_RIEUL, T_RIEUL, T_RIEUL],
+                [u'-을세', COND_NOT_RIEUL, '']],
+      'after': ['#용언', '#이다', '-으시-', '-었-'],
+      'notcond': ['#ㄷ불규칙', '#ㅂ불규칙', '#ㅅ불규칙', '#ㅎ불규칙'],
+    },
+    # ㄷ불규칙
+    TIKEUT_IRREGULAR_TYPICAL_CLASS(u'-을세', ['#용언']),
+    # ㅂ불규칙
+    PIEUP_IRREGULAR_TYPICAL_CLASS(u'-울세', ['#용언']),
+    # ㅅ불규칙
+    SIOS_IRREGULAR_TYPICAL_CLASS(u'-을세', ['#용언']),
+    # ㅎ불규칙
+    HIEUH_IRREGULAR_TYPICAL_CLASS(u'-%s세' % T_RIEUL, ['#용언']),
+]
+
+#### 연결: -요
+groups['-요'] = [
+    { 'rules': [[u'-요', '', '']],
+      'after': ['#이다', '아니다'],
+    },
+]
+
+#### 종결: -ㄹ라, -을라
+groups['-을라'] = [
+    { 'rules': [[u'-%s라' % T_RIEUL, COND_V_ALL, ''],
+                [u'-%s라' % T_RIEUL, T_RIEUL, T_RIEUL],
+                [u'-을라', COND_NOT_RIEUL, '']],
+      'after': ['#용언', '#이다', '-으시-', '-었-'],
+      'notcond': ['#ㄷ불규칙', '#ㅂ불규칙', '#ㅅ불규칙', '#ㅎ불규칙'],
+    },
+    # ㄷ불규칙
+    TIKEUT_IRREGULAR_TYPICAL_CLASS(u'-을라', ['#용언']),
+    # ㅂ불규칙
+    PIEUP_IRREGULAR_TYPICAL_CLASS(u'-울라', ['#용언']),
+    # ㅅ불규칙
+    SIOS_IRREGULAR_TYPICAL_CLASS(u'-을라', ['#용언']),
+    # ㅎ불규칙
+    HIEUH_IRREGULAR_TYPICAL_CLASS(u'-%s라' % T_RIEUL, ['#용언']),
+]
+
+#### 연결: -건대
+groups['-건대'] = [
+    { 'rules': [[u'-건대', '', ''],
+                [u'-컨대', COND_VOICED + u'하', u'하'], # 하건대 -> 건대
+                [u'-건대', COND_UNVOICED + u'하', u'하'], # 하건대 -> 건대
+                ],
+      'after': ['#동사'],
+    },
+]
